<?php
// $Id: guidance.admin.inc,v 1.2 2010/10/20 06:48:03 itarato Exp $


/**
 * Lists all the links of a guide.
 * This page for adding new links also.
 *
 * @param int $nid
 * @return string
 */
function guidance_guide_page($nid) {
  $guide = guidance_get_guide($nid);

  drupal_set_title(t('Guide: @title', array('@title' => _guidance_title_format($guide->title, '#'.$guide->nid))));

  $output = drupal_get_form('guidance_guide_links_form', $nid);

  $add = array('#value' => l(t('Add new link'), 'node/'.$nid.'/guidance/add'));
  $output .= drupal_render($add);

  return $output;
}


/**
 * Form for listing links
 */
function guidance_guide_links_form($form_original, $nid) {
  $links = guidance_get_guide_links($nid);
	$callback_array = module_invoke_all('guidance_callback');
	
  foreach((array)$links as $link) {
    $form['row'][$link['lid']] = array(
      'title' => array(
        '#value' => l(_guidance_title_format($link['title'], $link['url']), $link['url']),
      ),
      'description' => array(
        '#value' => check_plain($link['description']),
      ),
			'callback' => array(
				'#value' => $callback_array[$link['callback']]['info'],
			),
      'edit' => array(
        '#value' => l(t('Edit'), 'node/'.$nid.'/guidance/'.$link['lid'].'/edit'),
      ),
      'delete' => array(
        '#value' => l(t('Delete'), 'node/'.$nid.'/guidance/'.$link['lid'].'/delete'),
      ),
      'weight' => array(
        '#type' => 'weight',
        '#title' => t('Weight'),
        '#default_value' => 0,
        '#delta' => 10,
        '#attributes' => array(
          'class' => 'links-weight',
        ),
      ),
    );
  }

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save order'),
  );
  $form['#theme'] = 'guidance_guide_links_form';
  $form['#tree'] = TRUE;

  return $form;
}


/**
 * Submit for link form
 */
function guidance_guide_links_form_submit($form, $form_state) {
  foreach ((array)$form_state['values']['row'] as $lid => $link) {
    db_query('UPDATE {guidance_links} SET weight = %d WHERE lid = %d;', $link['weight'], $lid);
  }
}


/**
 * Theme for link listing on a guide
 */
function theme_guidance_guide_links_form($form) {
  $header = array(t('Link'), t('Description'), t('Callback'), NULL, t('Edit'), t('Delete'));
  $rows = array();
  foreach ((array)$form['row'] as $lid => $row) {
    //if (!is_numeric($lid)) continue;
		if (substr($lid, 0, 1) == '#') continue;
    $rows[] = array(
      'data' => array(
        drupal_render($form['row'][$lid]['title']),
        drupal_render($form['row'][$lid]['description']),
				drupal_render($form['row'][$lid]['callback']),
        drupal_render($form['row'][$lid]['weight']),
        drupal_render($form['row'][$lid]['edit']),
        drupal_render($form['row'][$lid]['delete']),
      ),
      'class' => 'draggable',
    );
  }

  drupal_add_tabledrag('links_table', 'order', 'sibling', 'links-weight');

  return theme_table($header, $rows, array('id' => 'links_table')).drupal_render($form);
}


/**
 * Editing/adding 1 link
 */
function guidance_link_form($form, $nid, $lid = NULL) {
  if (empty($lid)) {
    drupal_set_title(t('Add new link'));
  } else {
    drupal_set_title(t('Edit link'));
    $link = guidance_get_guide_link($lid);
  }

	$callback_array = module_invoke_all('guidance_callback');
	$callbacks = array();
	$callback_help = '';
	foreach ((array)$callback_array as $callback_key => $callback_info) {
		$callbacks[$callback_key] = $callback_info['info'];
		$callback_help .= '<strong>'.$callback_info['info'].':</strong><br/>'.$callback_info['help'].'<br/><br/>';
	}

  $form = array(
    'title' => array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => check_plain($link['title']),
      '#description' => t('Title of the guide.'),
    ),
    'url' => array(
      '#type' => 'textfield',
      '#title' => t('Url'),
      '#description' => t('The path this guide item links to. This can be an internal Drupal path such as node/add or an external URL such as http://drupal.org. Enter <front> to link to the front page.'),
      '#default_value' => $link['url'],
    ),
		'callback' => array(
			'#type' => 'select',
			'#title' => t('Callback function to verify if a link is done.'),
			'#description' => t('This has to be a valid function with parameters: [$lid].'),
			'#options' => $callbacks,
			'#default_value' => !empty($link['callback']) ? check_plain($link['callback']) : 'guidance_is_page_visited',
		),
		'callback_help' => array(
			'#type' => 'fieldset',
			'#title' => t('Help for callback functions'),
			'#collapsible' => TRUE,
			'#collapsed' => TRUE,
			'help_description' => array(
				'#prefix' => '<div>',
				'#suffix' => '</div>',
				'#value' => $callback_help,
			),
		),
    'description' => array(
      '#type' => 'textarea',
      '#title' => t('Detailed description about the current link'),
      '#description' => t('This description will be displayed in a separate block you have to enable.'),
      '#default_value' => check_plain($link['description']),
    ),
    'nid' => array(
      '#type' => 'hidden',
      '#value' => $nid,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => empty($lid) ? t('Add link') : t('Edit link'),
    ),
  );

  if (!empty($lid)) {
    $form['lid'] = array(
      '#type' => 'hidden',
      '#value' => check_plain($lid),
    );
  }

  return $form;
}


/**
 * Validation for the link form
 */
function guidance_link_form_validate($form, $form_state) {
  if (strlen($form_state['values']['url']) == 0) {
    form_set_error('url', t('Missing url.'));
  }
}


/**
 * Submit for the link form
 */
function guidance_link_form_submit($form, $form_submit) {
  $link = new stdClass();
  $link->title = check_plain($form_submit['values']['title']);
  $link->url = $form_submit['values']['url'];
  $link->nid = check_plain($form_submit['values']['nid']);
  $link->description = check_plain($form_submit['values']['description']);
  $link->callback = check_plain($form_submit['values']['callback']);

  if (!empty($form_submit['values']['lid'])) {
    $link->lid = $form_submit['values']['lid'];
    drupal_write_record('guidance_links', $link, array('lid'));
  } else {
    drupal_write_record('guidance_links', $link);
  }

  drupal_set_message(t('Link saved.'));
  drupal_goto('node/'.$form_submit['values']['nid'].'/guidance');
}


/**
 * Form for deleting a link.
 */
function guidance_link_delete_form($form, $lid) {
  $link = guidance_get_guide_link($lid);
  drupal_set_title(t('Deleting link: @title', array('@title' => _guidance_title_format($link['title'], $link['url']))));
  return array(
    'alert' => array(
      '#value' => t('Are you sure about deleting this link?'),
      '#suffix' => '<br/>',
    ),
    'lid' => array(
      '#type' => 'hidden',
      '#value' => $lid,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => 'Delete',
    ),
  );
}


/**
 * Submission for the link delete form.
 */
function guidance_link_delete_form_submit($form, $form_state) {
  $link = guidance_get_guide_link($form_state['values']['lid']);
  db_query('DELETE FROM {guidance_links} WHERE lid = %d;', $form_state['values']['lid']);
  drupal_set_message(t('Link deleted.'));
  drupal_goto('node/'.$link->nid.'/guidance');
}

